import sys
import hou
from datetime import datetime
from XMLShapeFileReader import *
from simpleBuilding import *

from controlVariables import *
from grid import *
from radial import *
from compositing import *
from geometry import *
from districts import *


def main(*args):

	controlVariables.path = args[0]

	hou.hipFile.clear() #clean the scene
	fileToRead = controlVariables.path + args[1]
	fileReader = XMLShapeFileReader(fileToRead)
	process(fileReader)
	makeHDA(hou.node('obj/geo1'))
	
	
def makeHDA(container):
	#First, collapse the container into a subnet
	citySubnet = container.collapseIntoSubnet(container.children(), subnet_name = 'citySubnet')
	#now, create the HDA from the subnet
	cityHDANode = citySubnet.createDigitalAsset(name = 'cityHDA',
                                description = 'city HDA',
                                ignore_external_references = True) 
	cityHDANode.moveToGoodPosition()
	# Get the nodeType definition of the new city asset. 
	cityDefinition = cityHDANode.type().definition()


	#======================================
	# Create Parameter Interface
	#======================================
	#For each param, it seems that the steps are:
	# 1 - create a parm template. Do not forget to provide the number of components... and the dafault values.
	# 2 - add that parm template to the HDA definition
	citySize = hou.FloatParmTemplate(name='citySize', label='City scale', 
		num_components=3, 
		default_value=(1,1,1)
		)
	cityDefinition.addParmTuple(citySize, in_folder=("Params",), create_missing_folders=True)
	
	
	#print hou.ch('/obj/geo1/citySubnet/finalcopy/ncy')
	if controlVariables.loadMap == None:
		numPrims = 1000
	else:
		numPrims = hou.node('/obj/geo1/citySubnet/finalcopy').parm('ncy').eval()
		
	visibleLotsMin = hou.IntParmTemplate(name='visibleLotsMin', label='Visible lots min',
		num_components=1,
		default_value=(0,),
		max = numPrims
		)
	cityDefinition.addParmTuple(visibleLotsMin, in_folder=("Params",), create_missing_folders=True)
	
	visibleLotsMax = hou.IntParmTemplate(name='visibleLotsMax', label='Visible lots max',
		num_components=1,
		default_value=(0,),
		max = numPrims
		)
	cityDefinition.addParmTuple(visibleLotsMax, in_folder=("Params",), create_missing_folders=True)

	if controlVariables.loadMap==None:
		lotSize = hou.FloatParmTemplate(name='lotSize', label='Lot width', 
			num_components=2, 
			default_value=(1.1,1.1),
			)
		cityDefinition.addParmTuple(lotSize, in_folder=("Params",), create_missing_folders=True)
		
		blockSize = hou.FloatParmTemplate(name='blockSize', label='Block scale', 
			num_components=2, 
			default_value=(float(controlVariables.blockSizex),float(controlVariables.blockSizey))
			)
		cityDefinition.addParmTuple(blockSize, in_folder=("Params",), create_missing_folders=True)

		if controlVariables.streetsAngle!='random':
			streetRotation = hou.FloatParmTemplate(name='streetRotation', label='Street Rotation',
				num_components=1,
				default_value=(float(controlVariables.streetsAngle),),
				min = 0,
				min_is_strict = True,
				max = 360,
				max_is_strict = True
				)
			cityDefinition.addParmTuple(streetRotation, in_folder=("Params",), create_missing_folders=True)	
	

	buildingSize = hou.FloatParmTemplate(name='buildingSize', label='Inner lot scale',
			num_components=2, 
			default_value=(float(controlVariables.lotsWidth),float(controlVariables.lotsWidth))
			)
	cityDefinition.addParmTuple(buildingSize, in_folder=("Params",), create_missing_folders=True)
	

	#=========================================
	# Link parameters with corresponding nodes
	#=========================================
	# Linking is simple: just add an expression referring to the parent channel
	scale = findSubNode(cityHDANode,'scale')
	scale.parm("sx").setExpression('ch("../citySizex")')
	scale.parm("sy").setExpression('ch("../citySizey")')
	scale.parm("sz").setExpression('ch("../citySizez")')
	
	scale2 = findSubNode(cityHDANode,'scale2')
	scale2.parm("sx").setExpression('ch("../citySizex")')
	scale2.parm("sy").setExpression('ch("../citySizey")')
	scale2.parm("sz").setExpression('ch("../citySizez")')
	
	visible = findSubNode(cityHDANode,'visibles')
	visible.parm("rangestart").setExpression('ch("../visibleLotsMin")')
	visible.parm("rangeend").setExpression('ch("../visibleLotsMax")')
	
	if controlVariables.loadMap==None:
		scaleLots = findSubNode(cityHDANode,'scaleLots')
		scaleLots.parm("sx").setExpression('ch("../lotSizex")')
		scaleLots.parm("sy").setExpression('ch("../lotSizey")')
		
		blockSize = findSubNode(cityHDANode,'blockSize')
		blockSize.parm("lsx").setExpression('ch("../blockSizex")')
		blockSize.parm("lsy").setExpression('ch("../blockSizey")')
		
		if controlVariables.streetsAngle!='random':
			rotation = findSubNode(cityHDANode,'rotation')
			rotation.parm("rz").setExpression('ch("../streetRotation")')

	buildingSize = findSubNode(cityHDANode,'buildingSize')
	buildingSize.parm("lsx").setExpression('ch("../buildingSizex")')
	buildingSize.parm("lsy").setExpression('ch("../buildingSizey")')

	
def findSubNode(searchSpace, nodeName):
	theNode = None
	for n in searchSpace.children():
		if (n.name() == nodeName):
			theNode = n
	return theNode



def process(fileReader):

	controlVariables.loadMap = None
	controlVariables.floor = None
	controlVariables.districtMap = None
	controlVariables.waterMap = None

	fileReader.initRules()
	while not fileReader.finishedRules():
		
		pre = fileReader.getPredecessor()
	  
		fileReader.initActions()
		while not fileReader.finishedActions():

			actionName = fileReader.currentActionName()
	
			if actionName == 'districts':
				controlVariables.districtMap = controlVariables.path + fileReader.readOptionalParameters([('districtMap',None)])[0]
				controlVariables.streetsAngle = fileReader.readOptionalParameters([('angle','random')])[0]
				controlVariables.distColor1, controlVariables.distColor2 = fileReader.readOptionalParameters([('colorDef1',None),('colorDef2',None)])
				#print "Color codes: color1=", controlVariables.distColor1, "color2=", controlVariables.distColor2

				numDist = len(fileReader.getParameters()) - 2

				i = 0
				while i <numDist:
					#print "Processing district", i, "..."
					fileReader.selectNextRule()
					fileReader.initActions()
					actionName = fileReader.currentActionName()

					if actionName == 'grid':
						controlVariables.gridFrequencyx, controlVariables.gridFrequencyy = fileReader.readCompulsoryParameters(['freqx','freqy'])
						controlVariables.gridCx, controlVariables.gridCy = fileReader.readCompulsoryParameters(['cy','cx'])
					elif actionName == 'radial':
						controlVariables.radialFrequency, controlVariables.radialAngularFrequency = fileReader.readCompulsoryParameters(['freqRadial','freqAngular'])
						controlVariables.radialCx, controlVariables.radialCy = fileReader.readCompulsoryParameters(['cy','cx'])
						#print "controlVariables: radialCx", controlVariables.radialCx, " radialCy", controlVariables.radialCy

					i = i+1
				

				d = districts()
				vex('surface')
				d.districts('surface')
				vex('filter')
				d.districts('filter')
	

			elif actionName == 'loadMap':
				controlVariables.loadMap = controlVariables.path + fileReader.readOptionalParameters([('file',None)])[0]
				controlVariables.floor = controlVariables.path + fileReader.readOptionalParameters([('floor',None)])[0]
				if controlVariables.floor == 'None':
					controlVariables.floor = None
				controlVariables.pattern = controlVariables.loadMap

			elif actionName == 'landUse':
				if fileReader.readCompulsoryParameters([('waterMap')])[0] != 'None':
					controlVariables.waterMap =  controlVariables.path + fileReader.readCompulsoryParameters([('waterMap')])[0]
				c = compositing()
				c.compositing()

			elif actionName == 'grid':
				controlVariables.pattern = 'grid'
				controlVariables.gridFrequencyx, controlVariables.gridFrequencyy = fileReader.readCompulsoryParameters(['freqx','freqy'])
				controlVariables.gridCx, controlVariables.gridCy = fileReader.readCompulsoryParameters(['cy','cx'])
	
				g = grid()
				vex('surface')
				g.createSurface('surface')
				vex('filter')
				g.createSurface('filter')

			elif actionName == 'radial':
				controlVariables.pattern = 'radial'
				controlVariables.radialFrequency, controlVariables.radialAngularFrequency = fileReader.readCompulsoryParameters(['freqRadial','freqAngular'])
				controlVariables.radialCx, controlVariables.radialCy = fileReader.readCompulsoryParameters(['cy','cx'])

				r = radial()
				vex('surface')
				r.createSurface('surface')
				vex('filter')
				r.createSurface('filter')

			elif actionName == 'divideStreets':
				controlVariables.streetsAngle = fileReader.readOptionalParameters([('angle','random')])[0]
				controlVariables.blockSizex, controlVariables.blockSizey = fileReader.readCompulsoryParameters([('blockSizex'),('blockSizey')])

			elif actionName == 'makeLots':
				controlVariables.lotsWidth = fileReader.readCompulsoryParameters(['width'])[0]
				
			elif actionName == 'createBuildings':
				texturePath = fileReader.readCompulsoryParameters(['texturePath'])[0]
			
			fileReader.selectNextAction()

		fileReader.selectNextRule()

	building = SimpleBuilding().createSimpleBuilding('buildingHDA', controlVariables.path + texturePath)
	g = geometry()
	g.createGeometry()
	
	
	for n in hou.node('/vex').children():
		n.moveToGoodPosition()
		

def vex(type):

	if type == 'surface':
		   sur = hou.node('/vex').createNode('surface') #create surface
		   sur.setName('CityPattern') #change its name
		   controlVariables.sur = sur
	else:
		   sur = hou.node('/vex').createNode('cop2filter')#create filter
		   sur.setName('myPattern') #change its name
		   controlVariables.sur = sur